#include<bits/stdc++.h>

using namespace std;

typedef long long ll;
typedef vector<ll> vi;
typedef vector<vi> vvi;
typedef vector<double> vd;
typedef vector<vd> vvd;

const ll INF = 1e18;
const int INF_INT = 1000000;
const double EPS = 1e-6;

struct Line {
    int r0, r1;
    double t0, d;
    int val;
};


bool check_line(int r0, int r1, double x0, double x1, double v, vvd& T, Line& res, bool speed_line = false) {
    if (x1 - x0 < v*(r1 - r0)) 
        return false;

    int lcross = -1;
    int ltouch = -1;
    for (int i = 0; i < r1; ++i) {
        if (i == r0) continue;

        double xi = x0 + (x1 - x0)*(i - r0)/(r1 - r0);

        int pos = lower_bound(T[i].begin(), T[i].end(), xi) - T[i].begin(); 

        if (pos < T[i].size() and abs(T[i][pos] - xi) < EPS) {
            ltouch = i;
        }
        else if (pos%2) {
            lcross = i;
        }
    }

    if (lcross > r0)
        return false;

    if (ltouch > lcross)
        return false;

    int rcross = (int)T.size();
    int rtouch = (int)T.size();
    for (int i = (int)T.size() -1; i >= r1; --i) {
        double xi = x0 + (x1 - x0)*(i - r0)/(r1 - r0);

        int pos = lower_bound(T[i].begin(), T[i].end(), xi) - T[i].begin(); 

        if (pos < T[i].size() and abs(T[i][pos] - xi) < EPS) {
            rtouch = i;
        }
        else if (pos%2) {
            rcross = i;
        }
    }

    if (speed_line and rtouch < rcross)
        return false;

    res.r0 = lcross + 1;
    res.r1 = rcross - 1;
    res.d = (x1 - x0)/(r1 - r0);
    res.t0 = x0 + res.d*(res.r0 - r0);
    res.val = INF_INT;

    return true;
}

void process(vector<Line>& D, int r0, int r1) {
    int n = D.size();
    vector<double> bef_d(n), aft_d(n);
    for (int i = 0; i < n; ++i) {
        bef_d[i] = D[i].t0 + D[i].d*(r0 - D[i].r0);
        aft_d[i] = D[i].t0 + D[i].d*(r1 - D[i].r0);
    }

    vector<int> ord_bef(n), ord_aft(n);
    for (int i = 0; i < n; ++i) ord_bef[i] = ord_aft[i] = i;

    sort(ord_bef.begin(), ord_bef.end(), [&](int i, int j) {
            return (abs(bef_d[i] - bef_d[j]) > EPS) ? bef_d[i] < bef_d[j] : aft_d[i] > aft_d[j];
        });
    sort(ord_aft.begin(), ord_aft.end(), [&](int i, int j) {
            return (abs(aft_d[i] - aft_d[j]) > EPS) ? aft_d[i] < aft_d[j] : bef_d[i] > bef_d[j] ;
        });

    vector<int> bef(n), aft(n);
    int it = 0;
    for (int oi: ord_bef) bef[oi] = it++;
    it = 0;
    for (int oi: ord_aft) aft[oi] = it++;

    vector<int> fen(n, INF_INT);
    auto get_min = [&](int p) {
        int res = INF_INT;
        for (; p >= 0; p = (p&(p+1))-1) res = min(res, fen[p]);
        return res;
    };
    auto set_val = [&](int p, int val) {
        for (; p < n; p |= (p+1)) fen[p] = min(fen[p], val);
    };

    vector<int> new_dist(n,INF_INT); 

    reverse(ord_bef.begin(), ord_bef.end());
    for (int oi: ord_bef) {
        new_dist[oi] = min(new_dist[oi], 1 + get_min(aft[oi]));
        set_val(aft[oi], D[oi].val);
    }



    fen = vector<int>(n, INF_INT);
    reverse(ord_aft.begin(), ord_aft.end());

    for (int oi: ord_aft) {
        new_dist[oi] = min(new_dist[oi], 1 + get_min(bef[oi]));
        set_val(bef[oi], D[oi].val);
    }

    for (int i = 0; i < n; ++i) {
       D[i].val = min(D[i].val, new_dist[i]); 
    }
}


int main() {
    int n, m;

    cin >> n >> m; 

    double s, v;
    cin >> s >> v;
    vvd T(m+2);

    for (int i = 0; i < n; ++i) {
        int a, b;
        int r;
        cin >> a >> b >> r;
        T[r].push_back(a);
        T[r].push_back(b);
    }


    T[0].push_back(0);
    T[m+1].push_back(s);

    vvd T2 = T;

    T[0].push_back(-INF);
    T[m+1].push_back(INF);

    for (int i = 0; i <= m+1; ++i) {
        sort(T[i].begin(), T[i].end());
    }

    vector<vector<Line> > L(m+2);

    for (int i = 0; i <= m+1; ++i) {
        for (auto p1 : T2[i]) {
            for (int j = i+1; j <= m+1; ++j) {
                for (auto p2 : T2[j]) {
                    Line line;
                    if (check_line(i, j, p1, p2, v, T, line)) {
                        L[line.r0].push_back(line);
                    }
                }
            }

            Line line;
            if (check_line(i, i+1, p1, p1 + v, v, T, line, true)) {
                L[line.r0].push_back(line);
            }
        }
    }

    for (auto& line : L[0]) {
        line.val = 0;
    }

    vector<Line> D;
    D = L[0];

    for (int i = 0; i <= m; ++i) {
        for (auto& line : L[i+1]) {
            D.push_back(line);
        }

        process(D, i, i+1);
        
        int j = 0;
        while (j < D.size()) {
            if (D[j].r1 <= i) {
                swap(D[j], D.back());
                D.pop_back();
            }
            else {
                ++j;
            }
        }
    }

    int res = INF_INT;
    for (auto line : D) {
        res = min(res, line.val);
    }
    if (res == INF_INT)
        res = -1;

    cout << res << endl;
}
