#include<bits/stdc++.h>

using namespace std;

typedef long long ll;
typedef vector<ll> vi;
typedef vector<vi> vvi;

const ll SCALE = 8*9*5*7*11;
const ll INF = 1e18;
const int INF_INT = 1000000;

struct Line {
    int r0, r1;
    ll t0, d;
    int val;
};

bool check_line(int r0, int r1, ll x0, ll x1, ll v, vvi& T, Line& res, bool speed_line = false) {
    if (x1 - x0 < v*(r1 - r0)) 
        return false;

    int lcross = -1;
    int ltouch = -1;
    for (int i = 0; i < r1; ++i) {
        if (i == r0) continue;

        ll xi = x0 + (x1 - x0)*(i - r0)/(r1 - r0);

        int pos = lower_bound(T[i].begin(), T[i].end(), xi) - T[i].begin(); 

        if (pos < T[i].size() and T[i][pos] == xi) {
            ltouch = i;
        }
        else if (pos%2) {
            lcross = i;
        }
    }

    if (lcross > r0)
        return false;

    if (ltouch > lcross)
        return false;

    int rcross = (int)T.size();
    int rtouch = (int)T.size();
    for (int i = (int)T.size() -1; i >= r1; --i) {
        ll xi = x0 + (x1 - x0)*(i - r0)/(r1 - r0);

        int pos = lower_bound(T[i].begin(), T[i].end(), xi) - T[i].begin(); 

        if (pos < T[i].size() and T[i][pos] == xi) {
            rtouch = i;
        }
        else if (pos%2) {
            rcross = i;
        }
    }

    if (speed_line and rtouch < rcross)
        return false;

    res.r0 = lcross + 1;
    res.r1 = rcross - 1;
    res.d = (x1 - x0)/(r1 - r0);
    res.t0 = x0 + res.d*(res.r0 - r0);
    res.val = INF_INT;

    return true;
}

void process(vector<Line>& D, int r) {
    sort(D.begin(), D.end(), [r](const Line& a, const Line& b)
            {
                ll xa = a.t0 + a.d*(r - a.r0);
                ll xb = b.t0 + b.d*(r - b.r0);

                return xa < xb;
            });

    int n = D.size();
    vector<int> V(n);
    for (int i = 0; i < n; ++i) V[i] = i;

    sort(V.begin(), V.end(), [r, &D](const int& a, const int& b)
            {
                ll xa = D[a].t0 + D[a].d*(r + 1 - D[a].r0);
                ll xb = D[b].t0 + D[b].d*(r + 1 - D[b].r0);

                return xa < xb;
            });

    vector<int> Vd(n);
    for (int i = 0; i < n; ++i) Vd[V[i]] = i;

    vector<int> DS(n);
    for (int i = 0; i < n; ++i) DS[i] = D[i].val;


    for (int i = 0; i < n; ++i) {
        for (int j = 0; j < i; ++j) {
            if (Vd[j] >= Vd[i]) {
                DS[j] = min(DS[j], D[i].val + 1);
            }
        }

        for (int j = i+1; j < n; ++j) {
            if (Vd[j] <= Vd[i]) {
                DS[j] = min(DS[j], D[i].val + 1);
            }
        }
    }

    for (int i = 0; i < n; ++i) D[i].val = DS[i];
}


int main() {
    int n, m;

    cin >> n >> m; 
    ll s, v;
    cin >> s >> v;

    vvi T(m+2);

    for (int i = 0; i < n; ++i) {
        ll a, b;
        int r;
        cin >> a >> b >> r;
        T[r].push_back(a*SCALE);
        T[r].push_back(b*SCALE);
    }

    
    v *= SCALE;

    T[0].push_back(0);
    T[m+1].push_back(s*SCALE);

    vvi T2 = T;

    T[0].push_back(-INF);
    T[m+1].push_back(INF);

    for (int i = 0; i <= m+1; ++i) {
        sort(T[i].begin(), T[i].end());
    }

    vector<vector<Line> > L(m+2);

    for (int i = 0; i <= m+1; ++i) {
        for (auto p1 : T2[i]) {
            for (int j = i+1; j <= m+1; ++j) {
                for (auto p2 : T2[j]) {
                    Line line;
                    if (check_line(i, j, p1, p2, v, T, line)) {
                        L[line.r0].push_back(line);
                    }
                }
            }

            Line line;
            if (check_line(i, i+1, p1, p1 + v, v, T, line, true)) {
                L[line.r0].push_back(line);
            }
        }
    }

    for (auto& line : L[0]) {
        line.val = 0;
    }

    vector<Line> D;
    D = L[0];

    for (int i = 0; i <= m; ++i) {
        for (auto& line : L[i+1]) {
            D.push_back(line);
        }

        process(D, i);
        
        int j = 0;
        while (j < D.size()) {
            if (D[j].r1 <= i) {
                swap(D[j], D.back());
                D.pop_back();
            }
            else {
                ++j;
            }
        }
    }

    int res = INF_INT;
    for (auto line : D) {
        res = min(res, line.val);
    }
    if (res == INF_INT)
        res = -1;

    cout << res << endl;
}
